<?php
namespace plugins\surveys\controllers;

use plugins\surveys\models\Question;
use plugins\surveys\models\Survey;
use plugins\surveys\models\Choice;
/**
 * Survey controller
 * @author Jérôme Brilland
 * @version 1.1.1
 * @copyright (c) 2022-2023, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class QuestionController extends \CRUDController {

    protected $modelClassName = 'plugins\\surveys\\models\\Question';
    
    public function doList() {
        $this->surveyId = $this->request->getParameters()->getInt( 'survey_id' );
        if ( empty( $this->surveyId ) ) {
            $this->forward( 'controllers\\ErrorController', 400 );
        }
        $this->where += ['survey_id' => $this->surveyId];
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'surveys.questions' );
        parent::doList();
        $this->questions = $this->models;
    }
    
    public function doSearch() {
        $keywords = strtolower( $this->request->getParameters()->getString( 'keywords' ) );
        $this->where = ['text[~]' => $keywords];
        $this->doList();
        $this->render( 'list.html' );
    }
    
    public function doMoveUp() {
        $question = Question::findOne( ['question_id', 'position', 'survey_id'], ['question_id' => $this->request->getAttribute( 'id' )] );
        $question->moveUp();
        $this->redirect( BASE_URL . 'surveys/question/list?survey_id=' . $question->survey_id . '&order=position' );
    }
    
    public function doMoveDown() {
        $question = Question::findOne( ['question_id', 'position', 'survey_id'], ['question_id' => $this->request->getAttribute( 'id' )] );
        $question->moveDown();
        $this->redirect( BASE_URL . 'surveys/question/list?survey_id=' . $question->survey_id . '&order=position' );
    }

    public function doAddform() {
        parent::doAddform();
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'surveys.add_question' );
        $surveyId = $this->request->getParameters()->getInt( 'survey_id' );
        if ( empty( $surveyId ) && empty( $this->model->survey_id ) ) {
            $this->forward( 'controllers\\ErrorController', 400 );
        }
        if ( empty( $this->model->survey_id ) ) {
            $this->model->survey_id = $surveyId;
        }
        if ( empty( $this->model->type ) ) {
            $this->model->type = $this->request->getParameters()->getWord( 'type' ) ?: 'short_answer';
        }
        if ( 'likert_scale' == $this->model->type && empty( $this->model->choices ) ) {
			$array = ['strongly_disagree', 'disagree', 'neither_agree_nor_disagree', 'agree', 'strongly_agree'];
			foreach( $array as $text ) {
				$this->model->addChoice( new Choice( ['text' => $text] ) );
			}
		}
    }

    public function doAdd() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        parent::doAdd();
    }

    public function doUpdateform() {
        parent::doUpdateform();
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'surveys.edit_question' );
    }

    public function doUpdate() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        parent::doUpdate();
    }

    protected function afterAdd() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'surveys.question_successfully_created' );
        $this->redirect( BASE_URL . 'surveys/question/list?survey_id=' . $this->model->survey_id );
    }

    protected function afterUpdate() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'surveys.question_successfully_updated' );
        $this->redirect( BASE_URL . 'surveys/question/list?survey_id=' . $this->model->survey_id );
    }

    public function doDelete() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        parent::doDelete();
    }

    protected function afterDelete() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'surveys.question_successfully_deleted' );
        $this->redirect( BASE_URL . 'surveys/question/list?survey_id=' . $this->model->survey_id );
    }
}
