<?php
namespace plugins\surveys\controllers;

use plugins\surveys\models\Survey;
use Medoo\Medoo;
/**
 * Survey controller
 * @author Jérôme Brilland
 * @version 1.0
 * @copyright (c) 2022, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class SurveyController extends \CRUDController {

    protected $modelClassName = 'plugins\\surveys\\models\\Survey';

    public function doList() {
        if ( 0 == Survey::count() ) {
            $this->redirect( BASE_URL . 'surveys/survey/add' );
        }
        $this->columns = ['survey.survey_id', 'survey.title', 'survey.status', 'survey.created_at', 'survey.updated_at', 'question_number' => Medoo::raw('COUNT(<question.question_id>)')];
        $this->join = ['[>]question' => 'survey_id'];
        $this->where = ['GROUP' =>['survey.survey_id', 'survey.title', 'survey.status', 'survey.created_at', 'survey.updated_at']];
        parent::doList();
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'surveys' );
        $this->surveys = $this->models;
    }
    
    public function doSearch() {
        $keywords = strtolower( $this->request->getParameters()->getString( 'keywords' ) );
        $this->where = ['OR' => ['title[~]' => $keywords, 'description[~]' => $keywords]];
        $this->doList();
        $this->render( 'list.html' );
    }

    public function doAddform() {
        parent::doAddform();
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'surveys.create_survey' );
    }

    public function doAdd() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        parent::doAdd();
    }

    public function doUpdateform() {
        parent::doUpdateform();
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'surveys.edit_survey' );
    }

    public function doUpdate() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        parent::doUpdate();
    }

    protected function afterAdd() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'surveys.survey_successfully_created' );
        if ( $this->request->getParameters()->getBoolean( 'continue', false ) ) {
            $this->redirect( BASE_URL . 'surveys/survey/update/' . $this->model->survey_id );
        } else {
            $this->redirect( BASE_URL . 'surveys/survey/list' );
        }
    }

    protected function afterUpdate() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'surveys.survey_successfully_updated' );
        if ( $this->request->getParameters()->getBoolean( 'continue', false ) ) {
            $this->redirect( BASE_URL . 'surveys/survey/update/' . $this->model->survey_id );
        } else {
            $this->redirect( BASE_URL . 'surveys/survey/list' );
        }
    }

    public function doDelete() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'surveys.manage_surveys' );
        parent::doDelete();
    }
    
    protected function afterDelete() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'surveys.survey_successfully_deleted' );
        $this->redirect( BASE_URL . 'surveys/survey/list' );
    }

    public function doShow() {
        parent::doShow();

        if ( empty( $this->survey) ) {
            \IoC::resolve( 'eventmediator' )->trigger( 'page_not_found', [$this, $this->request, $this->response] );
            $this->forward( 'controllers\\ErrorController', '404' );
        }
        
        $i18n = \IoC::resolve( 'i18n' );
        if ( !$this->survey->isOpen() ) {
            $this->flash['warning'] = $i18n->__( 'surveys.survey_closed' );
        }
    
        if ( \Registry::exists( 'errors' ) ) {
            $this->errors = \Registry::get( 'errors' );
        }
        
        $this->answers = $this->request->getParameter( 'answers', [] );
        
        $this->likertOptions = [1 => 'surveys.strongly_disagree', 'surveys.disagree', 'surveys.neither_agree_nor_disagree', 'surveys.agree', 'surveys.strongly_agree'];
        
        $settings = \IoC::resolve( 'settings' );
        $this->title = $this->survey->title . ' | ' . $settings['site_title'];

        $latte = \IoC::resolve( 'view' )->getLatte();
        $latte->getLoader()->addLoader( \IoC::resolve( 'latteDatabaseLoader') );

        $database = \IoC::resolve( 'database' );
        $settings = \IoC::resolve( 'settings' );

        $this->render( $database->get( 'template', ['name'], ['template_id' => $settings['surveys.survey_template'] ] )['name'] );
    }

}
