<?php
namespace plugins\surveys\services;

use plugins\surveys\models\Survey;
use plugins\surveys\models\Responder;
use plugins\surveys\models\Answer;
use plugins\surveys\models\Choice;
/**
 * @author Jérôme Brilland
 * @version 1.2
 * @copyright (c) 2022, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class ReplyToSurvey {

    private $surveyId;
    private $answers;

    /**
     * @param int $surveyId Survey id
     * @param array $answers Answers
     * @access public
     */
    public function __construct( int $surveyId, array $answers ) {
        $this->surveyId = $surveyId;
        $this->answers = $answers;
    }

    public function perform() {
        $eventMediator = \IoC::resolve( 'eventmediator' );

        $survey = Survey::findOne( $this->surveyId );
        if ( empty( $survey ) ) {
            $eventMediator->trigger( 'invalid_survey' );
            return;
        } else {
            if ( !$survey->isOpen() ) {
                $eventMediator->trigger( 'survey_closed' );
                return;
            }
        }

        $i18n = \IoC::resolve( 'i18n' );

        $errors = [];
        foreach( $survey->questions as $question ) {
            if ( $question->is_required && ( !array_key_exists( $question->question_id, $this->answers ) ) ) {
                $errors[$question->question_id] = $i18n->__( 'surveys.answer_is_required' );
                continue;
            }
            switch ( $question->type ) {
            case 'short_answer':
            case 'long_answer':
                if ( $question->is_required && !\Validator::ValidateNotEmpty( $this->answers[$question->question_id] ) ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.answer_is_required' );
                }
                if ( isset( $question->minlength ) && !\Validator::ValidateMinLength( $this->answers[$question->question_id], $question->minlength ) ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.too_short_answer', $question->minlength );
                }
                if ( !empty( $question->maxlength ) && !\Validator::ValidateMaxLength( $this->answers[$question->question_id], $question->maxlength ) ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.too_long_answer', $question->maxlength );
                }
                break;
            case 'single_choice':
            case 'likert_scale':
                $choices = array_column( Choice::findMany( ['choice_id'], ['question_id' => $question->question_id] ), 'choice_id' );
                if ( isset( $this->answers[$question->question_id] ) && !\Validator::validateInArray( $this->answers[$question->question_id], $choices ) ) {
                    throw new \UnexpectedValueException();
                }
                break;
            case 'multiple_choice':
                if ( !is_array( $this->answers[$question->question_id] ) ) {
                    throw new \UnexpectedValueException();
                }
                $choices = array_column( Choice::findMany( ['choice_id'], ['question_id' => $question->question_id] ), 'choice_id' );
                $this->answers[$question->question_id] = array_filter( $this->answers[$question->question_id] );
                foreach( $this->answers[$question->question_id] as $choiceId ) {
                    if ( !\Validator::validateInArray( $choiceId, $choices ) ) {
                        throw new \UnexpectedValueException();
                    }
                }
                //~ if ( $question->is_required && empty( $this->answers[$question->question_id] ) ) {
                //~ $errors[$question->question_id] = $i18n->__( 'surveys.answer_is_required' );
                //~ }
                if ( is_numeric( $question->min_choices ) && count( $this->answers[$question->question_id] ) < $question->min_choices ) {
                    if ( 1 == $question->min_choices ) {
                        $errors[$question->question_id] = $i18n->__( 'surveys.at_least_1' );
                    } else {
                        $errors[$question->question_id] = $i18n->__( 'surveys.at_least_x', $question->min_choices );
                    }
                }
                if ( is_numeric( $question->max_choices ) && count( $this->answers[$question->question_id] ) > $question->max_choices ) {
                    if ( 1 == $question->max_choices ) {
                        $errors[$question->question_id] = $i18n->__( 'surveys.at_most_1' );
                    } else {
                        $errors[$question->question_id] = $i18n->__( 'surveys.at_most_x', $question->max_choices );
                    }
                }
                break;
            case 'email':
                if ( $question->is_required && !\Validator::validateEmail( $this->answers[$question->question_id] ) ) {
                    $errors[$question->question_id] = $i18n->__( 'invalid_email' );
                }
                if ( $question->is_required && !\Validator::ValidateNotEmpty( $this->answers[$question->question_id] ) ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.answer_is_required' );
                }
                break;
            case 'numeric':
                if ( $question->only_integer && !\Validator::validateInteger( $this->answers[$question->question_id] ) ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.invalid_integer' );
                } else {
                    if ( !\Validator::validateNumber( $this->answers[$question->question_id] ) ) {
                        $errors[$question->question_id] = $i18n->__( 'surveys.invalid_number' );
                    }
                }
                if ( is_numeric( $question->min ) && $this->answers[$question->question_id] < $question->min ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.number_greater_than_or_equal_to', $question->min );
                }
                if ( is_numeric( $question->max ) && $this->answers[$question->question_id] > $question->max ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.number_less_than_or_equal_to', $question->max  );
                }
                if ( $question->is_required && !\Validator::ValidateNotEmpty( $this->answers[$question->question_id] ) ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.answer_is_required' );
                }
                break;
            case 'ranking':
                $choices = array_column( Choice::findMany( ['choice_id'], ['question_id' => $question->question_id] ), 'choice_id' );
                $values = range( 1, count( $choices ) );
                $this->answers[$question->question_id] = array_filter( $this->answers[$question->question_id] );
                if ( $question->is_required && empty( $this->answers[$question->question_id] ) ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.answer_is_required' );
                    continue;
                }
                if ( count( $this->answers[$question->question_id] ) < count( $choices ) ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.please_rank', 1, count( $choices ) );
                }
                foreach( $this->answers[$question->question_id] as $choiceId => $value ) {
                    if ( !\Validator::validateInArray( $choiceId, $choices ) ) {
                        throw new \UnexpectedValueException();
                    }
                    if ( !\Validator::validateInteger( $value ) || $value < 1 || $value > count( $choices ) ) {
                        throw new \UnexpectedValueException();
                    }
                    $index = array_search( $value, $values );
                    if ( false === $index ) {
                        $errors[$question->question_id] = $i18n->__( 'surveys.please_rank', 1, count( $choices ) );
                    }
                    unset( $values[$index] );
                }
                break;
            case 'date':
                if ( $question->is_required && !\Validator::ValidateNotEmpty( $this->answers[$question->question_id] ) ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.answer_is_required' );
                } elseif ( !\Validator::ValidateDate( $this->answers[$question->question_id] ) ) {
                    throw new \UnexpectedValueException();
                }
                break;
            case 'time':
                if ( $question->is_required && !\Validator::ValidateNotEmpty( $this->answers[$question->question_id] ) ) {
                    $errors[$question->question_id] = $i18n->__( 'surveys.answer_is_required' );
                } elseif ( !\Validator::ValidateTime( $this->answers[$question->question_id] ) ) {
                    throw new \UnexpectedValueException();
                }
                break;
            default:
            }
        }

        if ( !empty( $errors ) ) {
            $eventMediator->trigger( 'survey_response_errors', [$errors] );
            return;
        }

        $fingerprint = hash( 'ripemd160', $_SERVER['REMOTE_ADDR'] . $_SERVER['HTTP_USER_AGENT'] . $_SERVER['HTTP_ACCEPT_LANGUAGE'] );
        if ( Responder::exists( ['survey_id' => $this->surveyId, 'hashed_fingerprint' => $fingerprint ] ) ) {
            $eventMediator->trigger( 'already_answered', [$this->surveyId] );
            return;
        }
        $database = \IoC::resolve( 'database' );
        $result = null;

        $database->action( function( $database ) use ( $fingerprint, $survey, &$result ) {
            $responder = new Responder();
            $responder->survey_id = $this->surveyId;
            $responder->hashed_fingerprint = $fingerprint;
            $responder->save();

            foreach( $survey->questions as $question ) {
                switch ( $question->type ) {
                case 'short_answer':
                case 'long_answer':
                case 'email':
                case 'numeric':
                case 'date':
                case 'time':
                    $answer = new Answer();
                    $answer->question_id = $question->question_id;
                    $answer->responder_id = $responder->responder_id;
                    $answer->value = $this->answers[$question->question_id];
                    if ( !$answer->save() ) {
                        return false;
                    }
                    $result[] = $answer;
                    break;
                case 'single_choice':
                case 'likert_scale':
                    $answer = new Answer();
                    $answer->question_id = $question->question_id;
                    $answer->responder_id = $responder->responder_id;
                    $answer->choice_id = $this->answers[$question->question_id];
                    if ( !$answer->save() ) {
                        return false;
                    }
                    $result[] = $answer;
                    break;
                case 'multiple_choice':
                    foreach ( $this->answers[$question->question_id] as $choiceId ) {
                        $answer = new Answer();
                        $answer->question_id = $question->question_id;
                        $answer->responder_id = $responder->responder_id;
                        $answer->choice_id = $choiceId;
                        if ( !$answer->save() ) {
                            return false;
                        }
                        $result[] = $answer;
                    }
                    break;
                case 'ranking':
                    foreach ( $this->answers[$question->question_id] as $choiceId => $value ) {
                        $answer = new Answer();
                        $answer->question_id = $question->question_id;
                        $answer->responder_id = $responder->responder_id;
                        $answer->choice_id = $choiceId;
                        $answer->value = $value;
                        if ( !$answer->save() ) {
                            return false;
                        }
                        $result[] = $answer;
                    }
                    break;
                default:
                    return false;
                }
            }
        } );

        $eventMediator->trigger( 'answers_saved', [$result] );
    }
}
